@@ -17,9 +17,12 @@
 
 from functools import lru_cache
 from subprocess import run
+from tempfile import NamedTemporaryFile
+
+from PIL import Image
 
-from . import get_version
 from ..exceptions import MissingDependencyError
+from . import get_version
 
 
 @lru_cache(maxsize=1)
@@ -36,16 +39,31 @@ def available():
 
 
 def quantize(input_file, output_file, quality_min, quality_max):
-    args = [
-        'pngquant',
-        '--force',
-        '--skip-if-larger',
-        '--output',
-        output_file,
-        '--quality',
-        f'{quality_min}-{quality_max}',
-        '--',
-        input_file,
-    ]
-    proc = run(args)
-    proc.check_returncode()
+    if input_file.endswith('.jpg'):
+        with Image.open(input_file) as im, NamedTemporaryFile(suffix='.png') as tmp:
+            im.save(tmp)
+            args = [
+                'pngquant',
+                '--force',
+                '--skip-if-larger',
+                '--output',
+                output_file,
+                '--quality',
+                f'{quality_min}-{quality_max}',
+                '--',
+                tmp.name,
+            ]
+            run(args)
+    else:
+        args = [
+            'pngquant',
+            '--force',
+            '--skip-if-larger',
+            '--output',
+            output_file,
+            '--quality',
+            f'{quality_min}-{quality_max}',
+            '--',
+            input_file,
+        ]
+        run(args)
