@@ -1,4 +1,4 @@
-# © 2015-17 James R. Barlow: github.com/jbarlow83
+# © 2015-19 James R. Barlow: github.com/jbarlow83
 #
 # This file is part of OCRmyPDF.
 #
@@ -18,15 +18,16 @@
 import logging
 import os
 import shutil
-import sys
 from math import isclose
 from pathlib import Path
-from subprocess import DEVNULL, PIPE, Popen
+from subprocess import PIPE, run
 
 import PIL
 import pytest
 from PIL import Image
 
+import ocrmypdf
+import pikepdf
 from ocrmypdf.exceptions import ExitCode, MissingDependencyError
 from ocrmypdf.exec import ghostscript, qpdf, tesseract
 from ocrmypdf.leptonica import Pix
@@ -45,44 +46,41 @@ RENDERERS = ['hocr', 'sandwich']
 
 
 @pytest.fixture(scope='session')
-def spoof_tesseract_crash(tmpdir_factory):
-    return spoof(tmpdir_factory, tesseract='tesseract_crash.py')
+def spoof_tesseract_crash(tmp_path_factory):
+    return spoof(tmp_path_factory, tesseract='tesseract_crash.py')
 
 
 @pytest.fixture(scope='session')
-def spoof_tesseract_big_image_error(tmpdir_factory):
-    return spoof(tmpdir_factory, tesseract='tesseract_big_image_error.py')
+def spoof_tesseract_big_image_error(tmp_path_factory):
+    return spoof(tmp_path_factory, tesseract='tesseract_big_image_error.py')
 
 
 @pytest.fixture(scope='session')
-def spoof_no_tess_no_pdfa(tmpdir_factory):
-    return spoof(tmpdir_factory, tesseract='tesseract_noop.py', gs='gs_pdfa_failure.py')
-
-
-@pytest.fixture(scope='session')
-def spoof_no_tess_pdfa_warning(tmpdir_factory):
+def spoof_no_tess_no_pdfa(tmp_path_factory):
     return spoof(
-        tmpdir_factory, tesseract='tesseract_noop.py', gs='gs_feature_elision.py'
+        tmp_path_factory, tesseract='tesseract_noop.py', gs='gs_pdfa_failure.py'
     )
 
 
 @pytest.fixture(scope='session')
-def spoof_no_tess_gs_render_fail(tmpdir_factory):
+def spoof_no_tess_pdfa_warning(tmp_path_factory):
     return spoof(
-        tmpdir_factory, tesseract='tesseract_noop.py', gs='gs_render_failure.py'
+        tmp_path_factory, tesseract='tesseract_noop.py', gs='gs_feature_elision.py'
     )
 
 
 @pytest.fixture(scope='session')
-def spoof_no_tess_gs_raster_fail(tmpdir_factory):
+def spoof_no_tess_gs_render_fail(tmp_path_factory):
     return spoof(
-        tmpdir_factory, tesseract='tesseract_noop.py', gs='gs_raster_failure.py'
+        tmp_path_factory, tesseract='tesseract_noop.py', gs='gs_render_failure.py'
     )
 
 
 @pytest.fixture(scope='session')
-def spoof_tess_bad_utf8(tmpdir_factory):
-    return spoof(tmpdir_factory, tesseract='tesseract_badutf8.py')
+def spoof_no_tess_gs_raster_fail(tmp_path_factory):
+    return spoof(
+        tmp_path_factory, tesseract='tesseract_noop.py', gs='gs_raster_failure.py'
+    )
 
 
 def test_quick(spoof_tesseract_cache, resources, outpdf):
@@ -112,7 +110,7 @@ def test_deskew(spoof_tesseract_noop, resources, outdir):
     )
 
     pix = Pix.open(deskewed_png)
-    skew_angle, skew_confidence = pix.find_skew()
+    skew_angle, _skew_confidence = pix.find_skew()
 
     print(skew_angle)
     assert -0.5 < skew_angle < 0.5, "Deskewing failed"
@@ -120,8 +118,8 @@ def test_deskew(spoof_tesseract_noop, resources, outdir):
 
 def test_remove_background(spoof_tesseract_noop, resources, outdir):
     # Ensure the input image does not contain pure white/black
-    im = Image.open(resources / 'congress.jpg')
-    assert im.getextrema() != ((0, 255), (0, 255), (0, 255))
+    with Image.open(resources / 'congress.jpg') as im:
+        assert im.getextrema() != ((0, 255), (0, 255), (0, 255))
 
     output_pdf = check_ocrmypdf(
         resources / 'congress.jpg',
@@ -147,8 +145,8 @@ def test_remove_background(spoof_tesseract_noop, resources, outdir):
     )
 
     # The output image should contain pure white and black
-    im = Image.open(output_png)
-    assert im.getextrema() == ((0, 255), (0, 255), (0, 255))
+    with Image.open(output_png) as im:
+        assert im.getextrema() == ((0, 255), (0, 255), (0, 255))
 
 
 # This will run 5 * 2 * 2 = 20 test cases
@@ -164,7 +162,7 @@ def test_exotic_image(
     check_ocrmypdf(
         resources / pdf,
         outfile,
-        '-dc',
+        '-dc' if pytest.helpers.have_unpaper() else '-d',
         '-v',
         '1',
         '--output-type',
@@ -219,11 +217,11 @@ def test_skip_ocr(spoof_tesseract_cache, resources, outpdf):
     assert pdfinfo[0].has_text
 
 
-@pytest.helpers.needs_pdfminer
 def test_redo_ocr(spoof_tesseract_cache, resources, outpdf):
     in_ = resources / 'graph_ocred.pdf'
     before = PdfInfo(in_, detailed_page_analysis=True)
-    out = check_ocrmypdf(in_, outpdf, '--redo-ocr', env=spoof_tesseract_cache)
+    out = outpdf
+    out = check_ocrmypdf(in_, out, '--redo-ocr')
     after = PdfInfo(out, detailed_page_analysis=True)
     assert before[0].has_text and after[0].has_text
     assert (
@@ -282,8 +280,7 @@ def test_maximum_options(
         resources / 'multipage.pdf',
         outpdf,
         '-d',
-        '-c',
-        '-i',
+        '-ci' if pytest.helpers.have_unpaper() else None,
         '-f',
         '-k',
         '--oversample',
@@ -338,23 +335,23 @@ def test_force_ocr_on_pdf_with_no_images(spoof_tesseract_crash, resources, no_ou
     pytest.helpers.is_macos() and pytest.helpers.running_in_travis(),
     reason="takes too long to install language packs in Travis macOS homebrew",
 )
-def test_french(spoof_tesseract_cache, resources, outdir):
+def test_german(spoof_tesseract_cache, resources, outdir):
     # Produce a sidecar too - implicit test that system locale is set up
-    # properly
+    # properly. It is fine that we are testing -l deu on a French file because
+    # we are exercising the functionality not going for accuracy.
     sidecar = outdir / 'francais.txt'
     p, out, err = run_ocrmypdf(
         resources / 'francais.pdf',
         outdir / 'francais.pdf',
         '-l',
-        'fra',
+        'deu',  # more commonly installed
         '--sidecar',
         sidecar,
         env=spoof_tesseract_cache,
     )
-    print(os.environ)
-    assert (
-        p.returncode == ExitCode.ok
-    ), "This test may fail if Tesseract language packs are missing"
+    if 'deu' not in tesseract.languages():
+        pytest.xfail(reason="tesseract-deu language pack not installed")
+    assert p.returncode == ExitCode.ok, "Requires tesseract deu language pack"
 
 
 def test_klingon(resources, outpdf):
@@ -445,6 +442,7 @@ def test_tesseract_crash_autorotate(spoof_tesseract_crash, resources, no_outpdf)
 
 
 @pytest.mark.parametrize('renderer', RENDERERS)
+@pytest.mark.slow
 def test_tesseract_image_too_big(
     renderer, spoof_tesseract_big_image_error, resources, outpdf
 ):
@@ -541,86 +539,18 @@ def test_jbig2_passthrough(spoof_tesseract_cache, resources, outpdf):
         'hocr',
         env=spoof_tesseract_cache,
     )
-
     out_pageinfo = PdfInfo(out)
     assert out_pageinfo[0].images[0].enc == Encoding.jbig2
 
 
-def test_stdin(spoof_tesseract_noop, ocrmypdf_exec, resources, outpdf):
-    input_file = str(resources / 'francais.pdf')
-    output_file = str(outpdf)
-
-    # Runs: ocrmypdf - output.pdf < testfile.pdf
-    with open(input_file, 'rb') as input_stream:
-        p_args = ocrmypdf_exec + ['-', output_file]
-        p = Popen(
-            p_args,
-            close_fds=True,
-            stdout=PIPE,
-            stderr=PIPE,
-            stdin=input_stream,
-            env=spoof_tesseract_noop,
-        )
-        out, err = p.communicate()
-
-        assert p.returncode == ExitCode.ok
-
-
-def test_stdout(spoof_tesseract_noop, ocrmypdf_exec, resources, outpdf):
-    input_file = str(resources / 'francais.pdf')
-    output_file = str(outpdf)
-
-    # Runs: ocrmypdf francais.pdf - > test_stdout.pdf
-    with open(output_file, 'wb') as output_stream:
-        p_args = ocrmypdf_exec + [input_file, '-']
-        p = Popen(
-            p_args,
-            close_fds=True,
-            stdout=output_stream,
-            stderr=PIPE,
-            stdin=DEVNULL,
-            env=spoof_tesseract_noop,
-        )
-        out, err = p.communicate()
-
-        assert p.returncode == ExitCode.ok
-
-    assert qpdf.check(output_file, log=None)
-
-
-@pytest.mark.skipif(
-    sys.version_info[0:3] >= (3, 6, 4), reason="issue fixed in Python 3.6.4"
-)
-def test_closed_streams(spoof_tesseract_noop, ocrmypdf_exec, resources, outpdf):
-    input_file = str(resources / 'francais.pdf')
-    output_file = str(outpdf)
-
-    def evil_closer():
-        os.close(0)
-        os.close(1)
-
-    p_args = ocrmypdf_exec + [input_file, output_file]
-    p = Popen(  # pylint: disable=subprocess-popen-preexec-fn
-        p_args,
-        close_fds=True,
-        stdout=None,
-        stderr=PIPE,
-        stdin=None,
-        env=spoof_tesseract_noop,
-        preexec_fn=evil_closer,
-    )
-    out, err = p.communicate()
-    print(err.decode())
-    assert p.returncode == ExitCode.ok
-
-
 def test_masks(spoof_tesseract_noop, resources, outpdf):
-    p, out, err = run_ocrmypdf(
-        resources / 'masks.pdf', outpdf, env=spoof_tesseract_noop
+    assert (
+        ocrmypdf.ocr(
+            resources / 'masks.pdf', outpdf, tesseract_env=spoof_tesseract_noop
+        )
+        == ExitCode.ok
     )
 
-    assert p.returncode == ExitCode.ok
-
 
 def test_linearized_pdf_and_indirect_object(spoof_tesseract_noop, resources, outpdf):
     check_ocrmypdf(resources / 'epson.pdf', outpdf, env=spoof_tesseract_noop)
@@ -683,6 +613,7 @@ language_model_penalty_non_freq_dict_word 0
     )
 
 
+@pytest.mark.slow  # This test sometimes times out in CI
 @pytest.mark.parametrize('renderer', RENDERERS)
 def test_tesseract_config_notfound(renderer, resources, outdir):
     cfg_file = outdir / 'nofile.cfg'
@@ -721,26 +652,11 @@ THIS FILE IS INVALID
     assert p.returncode == ExitCode.invalid_config
 
 
-@pytest.mark.skipif(tesseract.v4(), reason='arg has no effect in 4.0-beta1')
-def test_user_words(resources, outdir):
+@pytest.mark.skipif(not tesseract.has_user_words(), reason='not functional until 4.1.0')
+def test_user_words_ocr(resources, outdir):
+    # Does not actually test if --user-words causes output to differ
     word_list = outdir / 'wordlist.txt'
-    sidecar_before = outdir / 'sidecar_before.txt'
-    sidecar_after = outdir / 'sidecar_after.txt'
-
-    # Don't know how to make this test pass on various versions and platforms
-    # so weaken to merely testing that the argument is accepted
-    consistent = False
-
-    if consistent:
-        check_ocrmypdf(
-            resources / 'crom.png',
-            outdir / 'out.pdf',
-            '--image-dpi',
-            150,
-            '--sidecar',
-            sidecar_before,
-        )
-        assert 'cromulent' not in sidecar_before.open().read()
+    sidecar_after = outdir / 'sidecar.txt'
 
     with word_list.open('w') as f:
         f.write('cromulent\n')  # a perfectly cromulent word
@@ -756,9 +672,6 @@ def test_user_words(resources, outdir):
         word_list,
     )
 
-    if consistent:
-        assert 'cromulent' in sidecar_after.open().read()
-
 
 def test_form_xobject(spoof_tesseract_noop, resources, outpdf):
     check_ocrmypdf(
@@ -780,10 +693,10 @@ def test_pagesize_consistency(renderer, resources, outpdf):
         outpdf,
         '--pdf-renderer',
         renderer,
-        '--clean',
+        '--clean' if pytest.helpers.have_unpaper() else None,
         '--deskew',
         '--remove-background',
-        '--clean-final',
+        '--clean-final' if pytest.helpers.have_unpaper() else None,
     )
 
     after_dims = first_page_dimensions(outpdf)
@@ -851,22 +764,21 @@ def test_compression_preserved(
             '-',
             output_file,
         ]
-        p = Popen(
+        p = run(
             p_args,
-            close_fds=True,
             stdout=PIPE,
             stderr=PIPE,
             stdin=input_stream,
+            universal_newlines=True,
             env=spoof_tesseract_noop,
         )
-        out, err = p.communicate()
 
         if im.mode in ('RGBA', 'LA'):
             # If alpha image is input, expect an error
-            assert p.returncode != ExitCode.ok and b'alpha' in err
+            assert p.returncode != ExitCode.ok and 'alpha' in p.stderr
             return
 
-        assert p.returncode == ExitCode.ok, err.decode('utf-8')
+        assert p.returncode == ExitCode.ok, p.stderr
 
     pdfinfo = PdfInfo(output_file)
 
@@ -880,6 +792,7 @@ def test_compression_preserved(
         assert pdfimage.color == Colorspace.rgb, "Colorspace changed"
     elif im.mode.startswith('L'):
         assert pdfimage.color == Colorspace.gray, "Colorspace changed"
+    im.close()
 
 
 @pytest.mark.parametrize(
@@ -912,17 +825,15 @@ def test_compression_changed(
             '-',
             output_file,
         ]
-        p = Popen(
+        p = run(
             p_args,
-            close_fds=True,
             stdout=PIPE,
             stderr=PIPE,
             stdin=input_stream,
+            universal_newlines=True,
             env=spoof_tesseract_noop,
         )
-        out, err = p.communicate()
-
-        assert p.returncode == ExitCode.ok, err
+        assert p.returncode == ExitCode.ok, p.stderr
 
     pdfinfo = PdfInfo(output_file)
 
@@ -943,10 +854,11 @@ def test_compression_changed(
         assert pdfimage.color == Colorspace.rgb, "Colorspace changed"
     elif im.mode.startswith('L'):
         assert pdfimage.color == Colorspace.gray, "Colorspace changed"
+    im.close()
 
 
 def test_sidecar_pagecount(spoof_tesseract_cache, resources, outpdf):
-    sidecar = outpdf + '.txt'
+    sidecar = outpdf.with_suffix('.txt')
     check_ocrmypdf(
         resources / 'multipage.pdf',
         outpdf,
@@ -970,7 +882,7 @@ def test_sidecar_pagecount(spoof_tesseract_cache, resources, outpdf):
 
 
 def test_sidecar_nonempty(spoof_tesseract_cache, resources, outpdf):
-    sidecar = outpdf + '.txt'
+    sidecar = outpdf.with_suffix('.txt')
     check_ocrmypdf(
         resources / 'ccitt.pdf', outpdf, '--sidecar', sidecar, env=spoof_tesseract_cache
     )
@@ -997,36 +909,10 @@ def test_pdfa_n(spoof_tesseract_cache, pdfa_level, resources, outpdf):
     assert pdfa_info['conformance'] == f'PDF/A-{pdfa_level}B'
 
 
-@pytest.mark.skipif(sys.version_info >= (3, 7, 0), reason='better utf-8')
-def test_bad_locale():
-    env = os.environ.copy()
-    env['LC_ALL'] = 'C'
-
-    p, out, err = run_ocrmypdf('a', 'b', env=env)
-    assert out == '', "stdout not clean"
-    assert p.returncode != 0
-    assert 'configured to use ASCII as encoding' in err, "should whine"
-
-
-@pytest.mark.parametrize('renderer', RENDERERS)
-def test_bad_utf8(spoof_tess_bad_utf8, renderer, resources, no_outpdf):
-    p, out, err = run_ocrmypdf(
-        resources / 'ccitt.pdf',
-        no_outpdf,
-        '--pdf-renderer',
-        renderer,
-        env=spoof_tess_bad_utf8,
-    )
-
-    assert out == '', "stdout not clean"
-    assert p.returncode != 0
-    assert 'not utf-8' in err, "should whine about utf-8"
-    assert '\\x96' in err, 'should repeat backslash encoded output'
-
-
 @pytest.mark.skipif(
     PIL.__version__ < '5.0.0', reason="Pillow < 5.0.0 doesn't raise the exception"
 )
+@pytest.mark.slow
 def test_decompression_bomb(resources, outpdf):
     p, out, err = run_ocrmypdf(resources / 'hugemono.pdf', outpdf)
     assert 'decompression bomb' in err
@@ -1051,14 +937,6 @@ def test_text_curves(spoof_tesseract_noop, resources, outpdf):
     assert len(info.pages[0].images) != 0, "force did not rasterize"
 
 
-def test_dev_null(spoof_tesseract_noop, resources):
-    p, out, err = run_ocrmypdf(
-        resources / 'trivial.pdf', os.devnull, '--force-ocr', env=spoof_tesseract_noop
-    )
-    assert p.returncode == 0, "could not send output to /dev/null"
-    assert len(out) == 0, "wrote to stdout"
-
-
 def test_output_is_dir(spoof_tesseract_noop, resources, outdir):
     p, out, err = run_ocrmypdf(
         resources / 'trivial.pdf', outdir, '--force-ocr', env=spoof_tesseract_noop
@@ -1094,3 +972,45 @@ def test_version_check():
 
     with pytest.raises(MissingDependencyError):
         get_version('echo')
+
+
+@pytest.mark.parametrize(
+    'threshold, optimize, output_type, expected',
+    [
+        [1.0, 0, 'pdfa', False],
+        [1.0, 0, 'pdf', False],
+        [0.0, 0, 'pdfa', True],
+        [0.0, 0, 'pdf', True],
+        [1.0, 1, 'pdfa', False],
+        [1.0, 1, 'pdf', False],
+        [0.0, 1, 'pdfa', True],
+        [0.0, 1, 'pdf', True],
+    ],
+)
+def test_fast_web_view(
+    spoof_tesseract_noop, resources, outpdf, threshold, optimize, output_type, expected
+):
+    check_ocrmypdf(
+        resources / 'trivial.pdf',
+        outpdf,
+        '--fast-web-view',
+        threshold,
+        '--optimize',
+        optimize,
+        '--output-type',
+        output_type,
+        env=spoof_tesseract_noop,
+    )
+    with pikepdf.open(outpdf) as pdf:
+        assert pdf.is_linearized == expected
+
+
+def test_image_dpi_not_image(caplog, spoof_tesseract_noop, resources, outpdf):
+    check_ocrmypdf(
+        resources / 'trivial.pdf',
+        outpdf,
+        '--image-dpi',
+        '100',
+        env=spoof_tesseract_noop,
+    )
+    assert '--image-dpi is being ignored' in caplog.text
