@@ -140,8 +140,8 @@ def test_autorotate_threshold(
         '--rotate-pages-threshold',
         threshold,
         '-r',
-        '-v',
-        '1',
+        # '-v',
+        # '1',
         env=spoof_tesseract_cache,
     )
 
@@ -224,12 +224,12 @@ def test_rotate_deskew_timeout(resources, outdir):
 @pytest.mark.parametrize('image_angle', (0, 90, 180, 270))
 def test_rotate_page_level(image_angle, page_angle, resources, outdir):
     def make_rotate_test(prefix, image_angle, page_angle):
-        im = Image.open(fspath(resources / 'typewriter.png'))
-        if image_angle != 0:
-            ccw_angle = -image_angle % 360
-            im = im.transpose(getattr(Image, f'ROTATE_{ccw_angle}'))
         memimg = BytesIO()
-        im.save(memimg, format='PNG')
+        with Image.open(fspath(resources / 'typewriter.png')) as im:
+            if image_angle != 0:
+                ccw_angle = -image_angle % 360
+                im = im.transpose(getattr(Image, f'ROTATE_{ccw_angle}'))
+            im.save(memimg, format='PNG')
         memimg.seek(0)
         mempdf = BytesIO()
         img2pdf.convert(
@@ -263,12 +263,12 @@ def test_rotate_page_level(image_angle, page_angle, resources, outdir):
     assert check_monochrome_correlation(outdir, reference, 1, out, 1) > 0.2
 
 
-def test_tesseract_orientation(resources, tmpdir):
+def test_tesseract_orientation(resources, tmp_path):
     pix = leptonica.Pix.open(resources / 'crom.png')
     pix_rotated = pix.rotate_orth(2)  # 180 degrees clockwise
-    pix_rotated.write_implied_format(tmpdir / '000001.png')
+    pix_rotated.write_implied_format(tmp_path / '000001.png')
 
-    log = Mock()
+    log = logging.getLogger()
     tesseract.get_orientation(  # Test results of this are unreliable
-        tmpdir / '000001.png', engine_mode='3', timeout=10, log=log
+        tmp_path / '000001.png', engine_mode='3', timeout=10, log=log
     )
